@@ -40,6 +40,7 @@ extern FLAG cansuspendmyself;
 extern void suspendmyself _((void));
 extern FLAG tty_closed;
 extern FLAG hup;	/* set when SIGHUP is caught */
+extern FLAG intr_char;	/* set when SIGINT is caught */
 
 /* i/o internal */
 extern int input_fd;	/* terminal input file descriptor */
@@ -48,7 +49,7 @@ extern int inputreadyafter _((int msec));
 
 /* tty handling */
 extern void raw_mode _((FLAG state));
-extern void get_term _((void));
+extern void get_term _((char * TERM));
 extern void getwinsize _((void));
 
 /* output handling */
@@ -96,7 +97,7 @@ extern void set_cursor _((int, int));
 /* terminal screen attributes handling */
 extern void disp_normal _((void));
 extern void disp_selected _((FLAG bg, FLAG border));
-extern void disp_colour _((int));
+extern void disp_colour _((int, FLAG for_256_colours));
 extern void disp_cursor _((FLAG));
 extern void reverse_on _((void));
 extern void reverse_off _((void));
@@ -140,8 +141,18 @@ extern void menuitem_on _((void));
 extern void menuitem_off _((void));
 extern void set_colour_token _((int));
 
+extern char set_ansi7 [];
 extern char restore_ansi7 [];
-extern FLAG do_restore_ansi7;
+extern FLAG redefined_ansi7;
+extern char set_ansi2 [];
+extern char restore_ansi2 [];
+extern FLAG redefined_ansi2;
+extern char set_ansi3 [];
+extern char restore_ansi3 [];
+extern FLAG redefined_ansi3;
+extern char set_curscolr [];
+extern char restore_curscolr [];
+extern FLAG redefined_curscolr;
 
 
 /* screen output */
@@ -223,6 +234,7 @@ extern int mouse_shift;		/* modifier status */
 extern FLAG report_release;	/* selectively suppress release event */
 extern FLAG window_focus;	/* does the window have mouse/keyboard focus? */
 extern int mouse_buttons_pressed;	/* count pressed buttons */
+extern mousebutton_type last_mouse_event;
 
 /* auxiliary menu functions */
 extern void menu_mouse_mode _((FLAG));
@@ -250,6 +262,7 @@ extern FLAG use_ascii_graphics;		/* use ASCII graphics for borders */
 extern FLAG use_vga_block_graphics;	/* charset is VGA with block graphics */
 extern FLAG use_pc_block_graphics;	/* alternate charset is VGA with block graphics */
 extern FLAG use_vt100_block_graphics;
+extern FLAG full_menu_bg;		/* full coloured background */
 extern FLAG use_appl_cursor;
 extern FLAG use_appl_keypad;
 extern char menu_border_style;
