@@ -8,6 +8,8 @@
 #include "io.h"
 #include "termprop.h"
 
+#define dont_debug_mouse
+
 
 #if ! defined (__TURBOC__) && ! defined (VAXC)
 #include <sys/time.h>
@@ -69,7 +71,8 @@ static
 FLAG sco_shift_mode = False;
 
 /* mouse handling */
-mousebutton_type mouse_button, mouse_prevbutton;
+mousebutton_type mouse_button;
+mousebutton_type mouse_prevbutton = movebutton;
 int mouse_xpos, mouse_ypos, mouse_prevxpos, mouse_prevypos;
 int mouse_shift = 0;		/* modifier status */
 FLAG report_release = False;	/* selectively suppress release event */
@@ -182,30 +185,21 @@ int
 get_digits (result)
   int * result;
 {
-  register int index;
-  register int count;
+  register int byte1;
+  register int value;
 
-  index = read1byte ();
-  if (index == quit_char) {
-	quit = True;
-  }
+  byte1 = read1byte ();
   * result = -1;
 /* Convert input to a decimal number */
-  count = 0;
-  while (index >= '0' && index <= '9' && quit == False) {
-	count *= 10;
-	count += index - '0';
-	* result = count;
-	index = read1byte ();
-	if (index == quit_char) {
-		quit = True;
-	}
+  value = 0;
+  while (byte1 >= '0' && byte1 <= '9') {
+	value *= 10;
+	value += byte1 - '0';
+	* result = value;
+	byte1 = read1byte ();
   }
 
-  if (quit) {
-	return quit_char;
-  }
-  return index;
+  return byte1;
 }
 
 
@@ -231,7 +225,7 @@ get_string_nokeymap (prompt, inbuf, statfl, term_input)
 
 
 /*======================================================================*\
-|*				Terminal input				*|
+|*			Terminal input					*|
 \*======================================================================*/
 
 #ifdef debug_keytrack
@@ -247,7 +241,7 @@ do_trace_keytrack (tag, c)
   } else if (c == FUNcmd) {
 	printf ("FUNcmd");
   } else {
-	printf ("%02X", c);
+	printf ("%02lX", c);
   }
   printf (" [keyshift %X]\n", keyshift);
 }
@@ -348,6 +342,7 @@ readchar ()
   while (report_release == False &&
 	 command (c) == MOUSEfunction && mouse_button == releasebutton)
   {
+	last_mouse_event = releasebutton;
 #ifdef debug_mouse
 	printf ("readchar: ignoring mouse release (report_release == False)\n");
 #endif
@@ -446,6 +441,13 @@ readcharacter_mapping (map_keyboard, return_unicode)
 		skipmouse = True;
 		return FUNcmd;
 	} else if (command (unichar) == MOUSEfunction) {
+		if (mouse_button != releasebutton 
+		    && last_delta_readchar > 100 && skip_1click) {
+			/* skip click only immediately after focus in */
+			skip_1click = False;
+			skip_1release = False;
+		}
+
 		if (mouse_button == releasebutton && skip_1release) {
 #ifdef debug_mouse
 			printf ("skipping 1 mouse release event\n");
@@ -516,12 +518,12 @@ readcharacter_mapping (map_keyboard, return_unicode)
 		return unichar;
 	}
   } else if (cjk_term && multichar (unichar)) {
-	if (text_encoding_tag == 'C' && unichar == 0x8E) {
+	if (unichar == 0x8E && text_encoding_tag == 'C') {
 		unichar = (unichar << 24)
 			| (readchar_nokeymap () << 16)
 			| (readchar_nokeymap () << 8)
 			| readchar_nokeymap ();
-	} else if (text_encoding_tag == 'J' && unichar == 0x8F) {
+	} else if (unichar == 0x8F && (text_encoding_tag == 'J' || text_encoding_tag == 'X')) {
 		unichar = (unichar << 16)
 			| (readchar_nokeymap () << 8)
 			| readchar_nokeymap ();
@@ -542,7 +544,7 @@ readcharacter_mapping (map_keyboard, return_unicode)
 	return unichar;
   } else {
 	if (mapped_term) {
-		if (! return_unicode && (cjk_text || mapped_text) && ! remap_chars ()) {
+		if (! return_unicode && (cjk_text || mapped_text) && ! remapping_chars ()) {
 #ifdef debug_mapped_keyboard
 			printf ("rc_m mapped (ret_u %d) -> %04X\n", return_unicode, unichar);
 #endif
@@ -646,10 +648,14 @@ readcharacter_allbuttons (map_keyboard)
 }
 
 
-/*-------------------------------------------------------------------------*/
+/*======================================================================*\
+|*			Mouse input					*|
+\*======================================================================*/
 
 #ifdef debug_mouse
 
+extern void do_trace_mouse _((char * tag));
+
 void
 do_trace_mouse (tag)
   char * tag;
@@ -1079,7 +1085,9 @@ CURSgetxy ()
 #endif
 
 
-/*-------------------------------------------------------------------------*/
+/*======================================================================*\
+|*			Character input					*|
+\*======================================================================*/
 
 /* max. length of function key sequence to be detected
    (depending on the fkeymap table, approx. 7),
@@ -1601,6 +1609,9 @@ read1byte ()
 #ifdef CURSES
 	ch = curs_readchar ();
 #else
+#ifdef debug_readbyte
+	printf ("reading byte from io (report_winchg %d)\n", report_winchg);
+#endif
 	if (report_winchg) {
 		ch = __readchar_report_winchg ();
 	} else {
@@ -1908,6 +1919,8 @@ _readchar ()
 	}
 # endif
 	return FUNcmd;
+  } else if (ch == ' ' && (keyshift & ctrl_mask)) {
+	return 0;	/* ctrl-Space -> NUL */
   } else {
 	q_put (ch);
   }
@@ -1940,7 +1953,7 @@ _readchar ()
 
 		ch = get1byte ();
 #ifdef debug_kbesc
-		printf ("<^[%s> + [%dms] %02X\n", ctrl_queue + 1, escape_delay, ch);
+		printf ("<^[%s> + [%dms] %02lX\n", ctrl_queue + 1, escape_delay, ch);
 #endif
 
 		/* accept ESC prefix as Alt modifier */
@@ -1949,7 +1962,7 @@ _readchar ()
 				ch = get1byte ();
 				keyshift |= alt_mask;
 #ifdef debug_kbesc
-				printf ("-> <^[%s> + ^[/%02X\n", ctrl_queue + 1, ch);
+				printf ("-> <^[%s> + ^[/%02lX\n", ctrl_queue + 1, ch);
 #endif
 			}
 		}
@@ -1964,7 +1977,7 @@ _readchar ()
 			}
 			ch = get1byte ();
 #ifdef debug_kbesc
-			printf ("-> <^[%s> + >?/%02X\n", ctrl_queue + 1, ch);
+			printf ("-> <^[%s> + >?/%02lX\n", ctrl_queue + 1, ch);
 #endif
 		}
 
@@ -2006,7 +2019,7 @@ check_ctrl_byte:
 			}
 			ch = get1byte ();
 #ifdef debug_kbesc
-			printf ("-> <^[%s> + 1-8/%02X\n", ctrl_queue + 1, ch);
+			printf ("-> <^[%s> + 1-8/%02lX\n", ctrl_queue + 1, ch);
 #endif
 		} else if (ch == ';' && second_esc_char == '1') {
 			ch = get1byte ();
@@ -2019,7 +2032,7 @@ check_ctrl_byte:
 			}
 			ch = get1byte ();
 #ifdef debug_kbesc
-			printf ("-> <^[%s> + ;/%02X\n", ctrl_queue + 1, ch);
+			printf ("-> <^[%s> + ;/%02lX\n", ctrl_queue + 1, ch);
 #endif
 		} else if (ch >= '0' && ch <= '9'
 			   && (second_esc_char == 'O' || 
@@ -2049,7 +2062,7 @@ check_ctrl_byte:
 			}
 			ch = get1byte ();
 #ifdef debug_kbesc
-			printf ("-> <^[%s> + 0-9/%02X\n", ctrl_queue + 1, ch);
+			printf ("-> <^[%s> + 0-9/%02lX\n", ctrl_queue + 1, ch);
 #endif
 		} else if (sco_shift_mode && second_esc_char == '['
 			   && ch >= '0' && ch <= '9') {
@@ -2071,7 +2084,7 @@ check_ctrl_byte:
 			} else if (ch == '1') {
 				ch = get1byte ();
 #ifdef debug_kbesc
-				printf ("-> <^[%s> + 1/%02X\n", ctrl_queue + 1, ch);
+				printf ("-> <^[%s> + 1/%02lX\n", ctrl_queue + 1, ch);
 #endif
 				if (ch >= '0' && ch <= '6') {
 					/* SCO: ESC [11M → ESC [M + alt etc */
