@@ -560,7 +560,7 @@ text_at (line, colpoi, targcol)
    and in the right order.
  */
 static
-void
+FLAG
 yank_text (fd, buf_status, 
 	start_line, start_textp, end_line, end_textp, 
 	do_remove, appending, do_rectangular_paste, start_end_reversed)
@@ -592,7 +592,7 @@ yank_text (fd, buf_status,
 
 /* Check file to hold buffer */
   if (fd == ERRORS) {
-	return;
+	return ERRORS;
   }
 
 /* Inform about running operation (to be seen in case it takes long) */
@@ -632,7 +632,7 @@ yank_text (fd, buf_status,
 	/* reject empty copy */
 	if (start_col == end_col) {
 		error ("Rectangular area empty");
-		return;
+		return ERRORS;
 	}
 
 	line_startp = textp = text_at (line, & col, start_col);
@@ -648,7 +648,7 @@ yank_text (fd, buf_status,
 		if (! do_rectangular_paste && line == end_line) {
 			error ("Internal error: passed end of text to be copied");
 			(void) close (fd);
-			return;
+			return ERRORS;
 		}
 
 		/* fill rectangular area */
@@ -661,17 +661,19 @@ yank_text (fd, buf_status,
 				if (writechar (fd, ' ') == ERRORS) {
 					error2 ("Write to buffer failed: ", serror ());
 					(void) close (fd);
-					return;
+					return ERRORS;
 				}
 			}
 		}
 
 		/* handle different line ends */
+		pasting = True;
 		return_len = write_lineend (fd, line->return_type, False);
+		pasting = False;
 		if (return_len == ERRORS) {
 			error2 ("Write to buffer failed: ", serror ());
 			(void) close (fd);
-			return;
+			return ERRORS;
 		}
 		lines_written ++;
 		if (line->return_type != lineend_NONE) {
@@ -737,7 +739,7 @@ yank_text (fd, buf_status,
 				if (writechar (fd, * up) == ERRORS) {
 					error2 ("Write to buffer failed: ", serror ());
 					(void) close (fd);
-					return;
+					return ERRORS;
 				}
 				up ++;
 				bytes_written ++;
@@ -759,7 +761,7 @@ yank_text (fd, buf_status,
 				if (writechar (fd, * buf) == ERRORS) {
 					error2 ("Write to buffer failed: ", serror ());
 					(void) close (fd);
-					return;
+					return ERRORS;
 				}
 				buf ++;
 				bytes_written ++;
@@ -771,7 +773,7 @@ yank_text (fd, buf_status,
 			if (writechar (fd, * textp) == ERRORS) {
 				error2 ("Write to buffer failed: ", serror ());
 				(void) close (fd);
-				return;
+				return ERRORS;
 			}
 			bytes_written ++;
 
@@ -824,11 +826,11 @@ yank_text (fd, buf_status,
   if (flush_filebuf (fd) == ERRORS) {
 	error2 ("Write to buffer failed: ", serror ());
 	(void) close (fd);
-	return;
+	return ERRORS;
   }
   if (close (fd) < 0) {
 	error2 ("Write to buffer failed: ", serror ());
-	return;
+	return ERRORS;
   }
   * buf_status = VALID;
 
@@ -869,6 +871,7 @@ yank_text (fd, buf_status,
 	: "^P"				/* mined paste */
 	);
   status_uni (text_buffer);
+  return FINE;
 }
 
 
@@ -887,6 +890,7 @@ delete_text_buf (start_line, start_textp, end_line, end_textp)
 {
   if (emacs_buffer) {
 	set_buffer_open (False);
+	(void)
 	yank_text (yankfile (WRITE, True), & yank_status, 
 			start_line, start_textp, end_line, end_textp, 
 			DELETE, True, False, FORWARD);
@@ -910,7 +914,10 @@ get_pasteline (fd, buffer, len)
   char buffer [maxLINElen];
   int * len;
 {
-  int ret = get_line (fd, buffer, len, False);
+  int ret;
+  pasting = True;
+  ret = get_line (fd, buffer, len, False);
+  pasting = False;
 
   if (ret == NO_INPUT || ret == ERRORS) {
 	return ret;
@@ -931,7 +938,7 @@ get_pasteline (fd, buffer, len)
 		char * ppoi = poi;
 
 		advance_utf8 (& poi);
-		if (ppoi + ulen != poi) {
+		if (ppoi + ulen != poi || (* ppoi & 0xC0) == 0x80) {
 			/* illegal UTF-8 value */
 			* npoi ++ = '';
 			prev_uc = 0;
@@ -1071,7 +1078,7 @@ insert_file (fd, stay_old_pos, from_text_file)
 	/* empty file */
 	return;
   } else if (ret == ERRORS) {
-	/* error message? */
+	/* sleep; "0 lines" error message? - no, nothing inserted */
 	return;
   }
 
@@ -1081,7 +1088,7 @@ insert_file (fd, stay_old_pos, from_text_file)
   } else if (ret == NUL_LINE) {
 	return_type = lineend_NUL;
   } else if (ret != NO_LINE) {
-	return_type = extract_lineend_type (line_buffer, len);
+	return_type = got_lineend;
   } else {
 	return_type = cur_line->return_type;
   }
@@ -1098,6 +1105,11 @@ insert_file (fd, stay_old_pos, from_text_file)
   trace_line ("first<", line);
   trace_rectangular_paste (("insert <%s>\n", line_buffer));
   if (insert_text (line, cur_text, line_buffer) == ERRORS) {
+/*	"Out of memory" error already shown, nothing inserted
+	sleep (2);
+	ring_bell ();
+	error ("Partial line inserted");
+*/
 	return;
   }
   trace_line ("first>", line);
@@ -1141,6 +1153,8 @@ insert_file (fd, stay_old_pos, from_text_file)
 		if (line->next == tail) {
 			LINE * new_line = line_insert_after (line, "\n", 1, line->return_type);
 			if (new_line == NIL_LINE) {
+				ring_bell ();
+				status_fmt2 ("Out of memory - ", "Insertion failed");
 				ret = ERRORS;
 				break;
 			} else {
@@ -1261,11 +1275,13 @@ insert_file (fd, stay_old_pos, from_text_file)
 		} else if (ret == NUL_LINE) {
 			return_type = lineend_NUL;
 		} else {
-			return_type = extract_lineend_type (line_buffer, len);
+			return_type = got_lineend;
 		}
 		trace_rectangular_paste (("line_insert <%s>\n", line_buffer));
 		new_line = line_insert_after (line, line_buffer, len, return_type);
 		if (new_line == NIL_LINE) {
+			ring_bell ();
+			status_fmt2 ("Out of memory - ", "Insertion failed");
 			ret = ERRORS;
 			break;
 		} else {
@@ -1363,7 +1379,10 @@ insert_file (fd, stay_old_pos, from_text_file)
   }
 
 /* If number of added lines >= REPORT_CHANGED_LINES, print the count */
-  if (line_count >= REPORT_CHANGED_LINES) {
+  if (ret == ERRORS || line_count >= REPORT_CHANGED_LINES) {
+	if (ret == ERRORS) {
+		ring_bell ();
+	}
 	status_line (dec_out ((long) line_count), " lines added");
   }
 }
@@ -1414,6 +1433,7 @@ paste_buffer (old_pos, use_clipboard)
 		error ("Cannot access clipboard");
 		return;
 	}
+	status_uni ("Pasting from Windows clipboard");
 	if (cur_line->return_type == lineend_LF) {
 		lineends_CRLFtoLF = True;	/* temporary, reset below */
 	}
@@ -1424,6 +1444,7 @@ paste_buffer (old_pos, use_clipboard)
 		error ("No inter window buffer present");
 		return;
 	}
+	status_uni ("Pasting from cross-session buffer");
   } else {
 	if ((fd = yankfile (READ, False)) == ERRORS) {
 		int e = geterrno ();
@@ -1434,6 +1455,7 @@ paste_buffer (old_pos, use_clipboard)
 		}
 		return;
 	}
+	status_uni ("Pasting");
 	if (append_flag) {
 		close_buffer ();
 	}
@@ -1813,7 +1835,7 @@ MARK ()
 	GOMA ();
   } else if (highlight_selection && mark_line == cur_line && mark_text == cur_text) {
 	clear_highlight_selection ();
-	status_uni ("Mark set -selection hidden- HOP Mark toggles rectangular");
+	status_uni ("Mark set -selection hidden- HOP Mark toggles rectangular");
   } else {
 	setMARK (False);
   }
@@ -1930,9 +1952,11 @@ yankie ()
 {
 #ifdef __CYGWIN__
   /* also copy to Windows clipboard */
+  status_uni ("Copying to Windows clipboard");
   if (copyfile (yank_file, "/dev/clipboard") != True) {
 	/* ignore error */
   }
+  status_uni (text_buffer);
 #endif
 
 #ifdef linkyank
@@ -1960,11 +1984,13 @@ yankie ()
   /*	resort to copying if the file system does not support hard links
   */
 #endif
+  status_uni ("Copying to cross-session buffer");
   if (copyfile (yank_file, yankie_file) != True) {
 	/* no error handling here as a message would inappropriately 
 	   obscure the original paste buffer copy information message 
 	   or an error message to the paste buffer copy function */
   }
+  status_uni (text_buffer);
 }
 
 /*
@@ -2000,17 +2026,21 @@ yank_block (remove, append)
 		return;
 	case SMALLER :
 		set_buffer_open (append);
-		yank_text (yankfile (WRITE, append), & yank_status, 
+		if (yank_text (yankfile (WRITE, append), & yank_status, 
 				mark_line, mark_text, cur_line, cur_text, 
-				remove, append, rectangular_paste_mode, FORWARD);
-		yankie ();
+				remove, append, rectangular_paste_mode, FORWARD)
+		    == FINE) {
+			yankie ();
+		}
 		break;
 	case BIGGER :
 		set_buffer_open (append);
-		yank_text (yankfile (WRITE, append), & yank_status, 
+		if (yank_text (yankfile (WRITE, append), & yank_status, 
 				cur_line, cur_text, mark_line, mark_text, 
-				remove, append, rectangular_paste_mode, REVERSE);
-		yankie ();
+				remove, append, rectangular_paste_mode, REVERSE)
+		    == FINE) {
+			yankie ();
+		}
 		break;
 	case SAME :
 		status_uni ("No text selected for Copy or Cut");
@@ -2031,11 +2061,13 @@ yank_HTML (remove)
 		error ("HTML tag selection failed");
 		return;
 	case SMALLER :
+		(void)
 		yank_text (htmlfile (WRITE, False), & html_status, 
 				mark_line, mark_text, cur_line, cur_text, 
 				remove, False, False, FORWARD);
 		break;
 	case BIGGER :
+		(void)
 		yank_text (htmlfile (WRITE, False), & html_status, 
 				cur_line, cur_text, mark_line, mark_text, 
 				remove, False, False, REVERSE);
@@ -2278,7 +2310,7 @@ do_update_selection_marks (select, pos_x)
 			} else if (line == mark_line) {
 				sel_end = mark_text;
 			} else {
-				sel_end = NIL_PTR;	/* EOL and beyond */
+				sel_end = NIL_PTR; /* end of line and beyond */
 			}
 			select_line (line, sel_begin, sel_end);
 			if (line == mark_line) {
