@@ -56,7 +56,7 @@ static char * substitute _((LINE * line, LINE * * line_poi, REGEX * program,
 static void search _((char * message, FLAG method));
 static void re_search _((void));
 static void prev_search _((void));
-static void do_search _((REGEX * program, FLAG method));
+static FLAG do_search _((REGEX * program, FLAG method));
 static LINE * match _((REGEX * program, character * string, FLAG method));
 static int line_check _((REGEX * program, char * str, LINE * from, FLAG method));
 static int check_string _((REGEX * program, character * str, LINE * from, int * expression));
@@ -216,7 +216,7 @@ RESreverse ()
 	hop_flag = 0;
 	search_for (prevexpr, reverse_method (prevmethod), True);
   } else {
-	do_search (lastprogram, reverse_method (lastmethod));
+	(void) do_search (lastprogram, reverse_method (lastmethod));
   }
 }
 
@@ -264,6 +264,25 @@ get_expression (message)
  * make_expression is only called by search_for and maintains its own, 
  * independant search program buffer
  */
+static
+REGEX *
+make_expression_this (expr, case_ignore, programpoi)
+  char * expr;
+  FLAG case_ignore;
+  REGEX * programpoi;
+{
+  if (compile (expr, programpoi, case_ignore) == ERRORS) {
+	/* Compile new expression */
+	return NIL_REG;
+  }
+
+  if (programpoi->status == REG_ERROR) {	/* Error during compiling */
+	error (programpoi->result.err_mess);
+	return NIL_REG;
+  }
+  return programpoi;
+}
+
 static
 REGEX *
 make_expression (expr, case_ignore)
@@ -272,16 +291,7 @@ make_expression (expr, case_ignore)
 {
   static REGEX program;		/* Program of expression */
 
-  if (compile (expr, & program, case_ignore) == ERRORS) {
-	/* Compile new expression */
-	return NIL_REG;
-  }
-
-  if (program.status == REG_ERROR) {	/* Error during compiling */
-	error (program.result.err_mess);
-	return NIL_REG;
-  }
-  return & program;
+  return make_expression_this (expr, case_ignore, & program);
 }
 
 /*
@@ -352,7 +362,7 @@ change (message, global, confirm)
 			move_address (program->start_ptr, ly);
 			display_flush ();
 			set_cursor_xy ();
-			status_uni ("Replace ? (y/n/ESC)");
+			status_uni ("Replace ? (y/n/ESC)");
 			c = prompt ("yn");
 			clear_status ();
 			if (c == 'y') {
@@ -622,7 +632,7 @@ substitute (line, line_poi,
 	line = line_insert_after (line, repl_start, repl_len, old_return_type);
 	if (line == NIL_LINE) {
 		ring_bell ();
-		error ("Substitution failed: cannot allocate more lines");
+		status_fmt2 ("Out of memory - ", "Substitution failed");
 		return NIL_PTR;
 	}
 	if (* repl_end == '\0') {
@@ -686,7 +696,7 @@ static
 void
 re_search ()
 {
-  do_search (lastprogram, lastmethod);
+  (void) do_search (lastprogram, lastmethod);
 }
 
 static
@@ -741,7 +751,27 @@ search_for (expr, method, case_ignore)
   if ((program = make_expression (expr, case_ignore)) == NIL_REG) {
 	return;
   }
-  do_search (program, method);
+  (void) do_search (program, method);
+}
+
+/*
+   Search for expression, remember it for next search.
+ */
+FLAG
+search_ini (expr, method, case_ignore)
+  char * expr;
+  FLAG method;
+  FLAG case_ignore;
+{
+  register REGEX * program;
+
+/* make the expression */
+  if ((program = make_expression (expr, case_ignore)) == NIL_REG) {
+	return False;
+  }
+  lastprogram = program;
+  lastmethod = method;
+  return do_search (program, method);
 }
 
 /*
@@ -764,7 +794,7 @@ search_expr (expr, method, case_ignore)
 
   lastprogram = program;
   lastmethod = method;
-  do_search (program, method);
+  (void) do_search (program, method);
 }
 
 /*
@@ -790,9 +820,10 @@ search_corresponding (expr, method, corresponding)
 #ifdef debug_corresponding_redraw
   char pflags [9999];
 #endif
+  static REGEX myprogram;
 
 /* make the expression */
-  if ((program = make_expression (expr, True)) == NIL_REG) {
+  if ((program = make_expression_this (expr, True, & myprogram)) == NIL_REG) {
 	return;
   }
 
@@ -943,7 +974,7 @@ search_it (program, method)
 }
 
 static
-void
+FLAG
 do_search (program, method)
   register REGEX * program;
   FLAG method;
@@ -952,11 +983,11 @@ do_search (program, method)
 
   if (method == NOT_VALID) {
 	error ("No previous search");
-	return;
+	return False;
   }
   if (program == NIL_REG) {
 	error ("No previous search expression");
-	return;
+	return False;
   }
 
   if (! visselect_keeponsearch) {
@@ -965,9 +996,10 @@ do_search (program, method)
 
   match_line = search_it (program, method);
   if (match_line == NIL_LINE) {
-	return;
+	return False;
   }
   move_address (program->start_ptr, find_y (match_line));
+  return True;
 }
 
 
@@ -978,7 +1010,7 @@ do_search (program, method)
 /* Opcodes for characters */
 #define NORMAL		0x0200
 #define DOT		0x0400
-#define EOLN		0x0800
+#define ENDOFLN		0x0800
 #define STAR		0x8000
 #define NEGATE		0x4000
 #define BRACKET		0x2000
@@ -1062,7 +1094,7 @@ print_expression (expr)
 	if (* expr & NORMAL)	strcat (s, "=");
 	if (* expr & NEGATE)	strcat (s, "!");
 	if (* expr & DOT)	strcat (s, ".");
-	if (* expr & EOLN)	strcat (s, "\\n");
+	if (* expr & ENDOFLN)	strcat (s, "\\n");
 	if (* expr & STAR)	strcat (s, "*");
 	if (* expr & BRACKET)	strcat (s, "[");
 	if (* expr & DONE)	strcat (s, "DONE");
@@ -1138,10 +1170,10 @@ compile (pat_start, program, case_ignore)
 		break;
 	case '$' :
 		/*
-		 * Only means EOLN if it is the last char of the pattern
+		 * Only means ENDOFLN if it is the last char of the pattern
 		 */
 		if (* pattern == '\0') {
-			* expression ++ = EOLN | DONE;
+			* expression ++ = ENDOFLN | DONE;
 			program->status |= END_LINE;
 			trace_expression ((exp_buffer));
 			return finished (program, expression);
@@ -1154,7 +1186,7 @@ compile (pat_start, program, case_ignore)
 		trace_expression ((exp_buffer));
 		return finished (program, expression);
 	case '\n' :
-		* expression ++ = (NORMAL | EOLN) + '\n';
+		* expression ++ = (NORMAL | ENDOFLN) + '\n';
 		break;
 	case '\\' :
 		/* If last char, it must! mean a normal '\' */
@@ -1163,14 +1195,14 @@ compile (pat_start, program, case_ignore)
 		} else {
 			c = * pattern ++;
 			if (c == 'n') {
-				* expression ++ = (NORMAL | EOLN) + 'n';
+				* expression ++ = (NORMAL | ENDOFLN) + 'n';
 			} else if (c == 'r') {
-				* expression ++ = (NORMAL | EOLN) + '\r';
+				* expression ++ = (NORMAL | ENDOFLN) + '\r';
 			} else if (c == 'R') {
-				* expression ++ = (NORMAL | EOLN) + 'R';
+				* expression ++ = (NORMAL | ENDOFLN) + 'R';
 			} else if (c == '0') {
 				/* NUL is represented as pseudo-line-end */
-				* expression ++ = (NORMAL | EOLN) + '\0';
+				* expression ++ = (NORMAL | ENDOFLN) + '\0';
 			} else {
 				* expression ++ = NORMAL + c;
 			}
@@ -1431,9 +1463,9 @@ match (program, string, method)
 	line = (method == FORWARD) ? line->next : line->prev;
 	if (line->text == NIL_PTR) {	/* header/tail */
 		if (method == FORWARD) {
-			status_uni (">>>>>>>> Search wrapped around end of file >>>>>>>>");
+			status_uni (">>>>>>>> Search wrapped around end of file >>>>>>>>");
 		} else {
-			status_uni ("<<<<<<<< Search wrapped around top of file <<<<<<<<");
+			status_uni ("<<<<<<<< Search wrapped around top of file <<<<<<<<");
 		}
 		wrapped_search = True;
 		continue;
@@ -1577,7 +1609,7 @@ check_string (program, string, from_line, expression)
 	}
 	expression ++;		/* Increment expr. */
 
-	if (opcode & EOLN) {
+	if (opcode & ENDOFLN) {
 	    if (* string == '\n') {
 		/* handle multi-line matching */
 		lineend_type rettype = from_line->return_type;
@@ -1618,7 +1650,7 @@ check_string (program, string, from_line, expression)
 		return NO_MATCH;
 	    }
 	} else
-	switch (opcode & ~EOLN) {
+	switch (opcode & ~ENDOFLN) {
 	case NORMAL :
 		/* this case only applies to single-byte characters 
 		   as multi-byte characters in the search pattern are 
@@ -1742,13 +1774,13 @@ check_string (program, string, from_line, expression)
 	   See also comments above about matching \n or \r on last line 
 	   which fails here (2nd check) as replacement would not work.
 	 */
-	if ((* expression & EOLN) && * string != '\n' && * string != '\0') {
+	if ((* expression & ENDOFLN) && * string != '\n' && * string != '\0') {
 		trace_undent ();
 		return NO_MATCH;
 	}
 	if ((char *) string == (char *) program->start_ptr
 		&& ! (program->status & BEGIN_LINE)
-		&& ! (* expression & EOLN))
+		&& ! (* expression & ENDOFLN))
 	{
 		trace_undent ();
 		return NO_MATCH;
