@@ -30,6 +30,7 @@ FLAG euc4_term = True;		/* does CJK terminal support EUC 4 byte ? */
 FLAG cjklow_term = True;	/* does CJK terminal support 81-9F range ? */
 int cjk_tab_width;		/* width of CJK TAB indicator */
 int cjk_lineend_width;		/* width of CJK line end indicator */
+int cjk_currency_width = 0;	/* width of CJK cent/pound/yen characters */
 
 /* basic data versions of Unicode; defaults overwritten by auto-detection */
 #ifdef older_defaults
@@ -42,6 +43,7 @@ int combining_data_version = U500;	/* Unicode 5.0 */
 
 /* special modes */
 int cjk_width_data_version = 0;	/* xterm CJK legacy width mode -cjk_width */
+FLAG hangul_jamo_extended = False;	/* U+D7B0... combining Jamo ? */
 
 /* specific behaviour of specific terminals */
 FLAG unassigned_single_width = False;	/* unassigned chars displayed single width? */
